"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.serializeType = serializeType;

var _core = require("@babel/core");

function serializeType(path, param) {
  const node = param.type === 'Identifier' || param.type === 'ClassProperty' ? param : param.type === 'TSParameterProperty' ? param.parameter.type === 'AssignmentPattern' && param.parameter.left.type === 'Identifier' ? param.parameter.left : param.parameter.type === 'Identifier' ? param.parameter : null : null;
  if (node == null) return _core.types.nullLiteral();
  if (!node.typeAnnotation || node.typeAnnotation.type !== 'TSTypeAnnotation') return _core.types.nullLiteral();
  const annotation = node.typeAnnotation.typeAnnotation;

  switch (annotation.type) {
    case 'TSTypeReference':
      /**
       * We need to save references to this type since it is going
       * to be used as a Value (and not just as a Type) here.
       *
       * This is resolved in main plugin method, calling
       * `path.scope.crawl()` which updates the bindings.
       */
      const reference = serializeReference(annotation.typeName);
      /**
       * We don't know if type is just a type (interface, etc.) or a concrete
       * value (class, etc.).
       * `typeof` operator allows us to use the expression even if it is not
       * defined, fallback is just `Object`.
       *
       * @todo Support more base types like number, lists, etc.
       */

      return _core.types.conditionalExpression(_core.types.binaryExpression('===', _core.types.unaryExpression('typeof', reference), _core.types.stringLiteral('undefined')), _core.types.identifier('Object'), _core.types.clone(reference));

    default:
      return _core.types.nullLiteral();
  }
}

function serializeReference(typeName) {
  if (typeName.type === 'Identifier') return _core.types.identifier(typeName.name);
  return _core.types.memberExpression(serializeReference(typeName.left), typeName.right);
}